@@ -2,7 +2,7 @@
 * netoutput.cpp: network output
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: netoutput.cpp,v 1.7 2002/03/25 14:49:33 bozo Exp $
+* $Id: netoutput.cpp,v 1.11 2002/08/09 13:42:32 tooney Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -34,11 +34,15 @@
 #include "../../core/core.h"
 #include "../../mpeg/mpeg.h"
 #include "../../mpeg/ts.h"
+#include "../../mpeg/rtp.h"
 #include "../../server/buffer.h"
 #include "../../server/output.h"
 
 #include "netoutput.h"
 
+#ifdef HAVE_NET_IF_H
+#include <net/if.h>
+#endif
 
 //******************************************************************************
 // C_NetOutput class
@@ -50,7 +54,7 @@
 // 
 //------------------------------------------------------------------------------
 C_NetOutput::C_NetOutput(const C_String& strChannelName)
-                : C_Output(TS_IN_ETHER), m_cSocketBuff(TS_IN_ETHER)
+                : C_Output(TS_IN_ETHER), m_cSocketBuff(TS_IN_ETHER+1)
 {
   C_Application* pApp = C_Application::GetApp();
   ASSERT(pApp);
@@ -65,8 +69,11 @@ C_NetOutput::C_NetOutput(const C_String& strChannelName)
   m_iTTL = strTTL.ToInt();
 
   // Init the buffer
-  for(int iIndex = 0; iIndex < TS_IN_ETHER; iIndex++)
+  // The first slot is reserved for the Rtp Header
+  m_cSocketBuff.SetSlotSize(0, RTP_HEADER_LEN);
+  for(int iIndex = 1; iIndex < TS_IN_ETHER + 1; iIndex++)
     m_cSocketBuff.SetSlotSize(iIndex, TS_PACKET_LEN);
+
 }
 
 
@@ -93,12 +100,12 @@ void C_NetOutput::OnInit()
   {
     // Allow the socket to send broadcast packets
     iOptVal = 1;
-    m_cSocket.SetOption(SOL_SOCKET, SO_BROADCAST, &iOptVal, sizeof(iOptVal));
+    m_cSocket.SetOption(SOL_SOCKET, SO_BROADCAST, (char *)&iOptVal, sizeof(iOptVal));
   }
 
   // Allow to use the ip/port couple more than once at a time to be
   // able to send several streams to a same client using the same port
-  m_cSocket.SetOption(SOL_SOCKET, SO_REUSEADDR, &iOptVal, sizeof(iOptVal));
+  m_cSocket.SetOption(SOL_SOCKET, SO_REUSEADDR, (char *)&iOptVal, sizeof(iOptVal));
 
   // Try to increase the size of the socket output buffer to 1/2MB (8Mb/s
   // during 1/2s) to avoid packet loss
@@ -107,7 +114,7 @@ void C_NetOutput::OnInit()
   {
     try
     {
-      m_cSocket.SetOption(SOL_SOCKET, SO_SNDBUF, &iOptVal, sizeof(iOptVal));
+      m_cSocket.SetOption(SOL_SOCKET, SO_SNDBUF, (char *)&iOptVal, sizeof(iOptVal));
       break;
     }
     catch(E_Exception e)
@@ -119,6 +126,21 @@ void C_NetOutput::OnInit()
     }
   }
 
+#ifdef HAVE_IPV6
+  if ((m_cSocket.GetDomain() == AF_INET6)&&
+	  (m_strType == "multicast")&&
+	  (m_strInterface != ""))
+  {
+	/* now get the ifindex for this interface */
+	/* and set the appropriate option */
+    int mcif = if_nametoindex(m_strInterface.GetString());
+	if (mcif == 0) perror("if_nametoindex()\n");
+	else m_cSocket.SetOption(IPPROTO_IPV6, IPV6_MULTICAST_IF,
+                                 (char *)&mcif, sizeof(mcif));
+  }
+#endif
+
+
 #ifdef HAVE_SO_BINDTODEVICE
   // If an interface is specified then bind to it
   // (Very useful when several interfaces are connected to the same subnet)
@@ -136,11 +158,26 @@ void C_NetOutput::OnInit()
   if(m_iTTL)
   {
 #ifndef _WIN32
-    if((m_strType == "unicast") || (m_strType == "broadcast"))
-      m_cSocket.SetOption(IPPROTO_IP, IP_TTL, &m_iTTL, sizeof(m_iTTL));
-    else if(m_strType == "multicast")
-      m_cSocket.SetOption(IPPROTO_IP, IP_MULTICAST_TTL,
-                          &m_iTTL, sizeof(m_iTTL));
+	if (m_cSocket.GetDomain() == AF_INET)
+	{
+      if((m_strType == "unicast") || (m_strType == "broadcast"))
+        m_cSocket.SetOption(IPPROTO_IP, IP_TTL, (char *)&m_iTTL, sizeof(m_iTTL));
+      else if(m_strType == "multicast")
+        m_cSocket.SetOption(IPPROTO_IP, IP_MULTICAST_TTL,
+                            (char *)&m_iTTL, sizeof(m_iTTL));
+    }
+#ifdef HAVE_IPV6
+	else if (m_cSocket.GetDomain() == AF_INET6)
+	{
+	    /* of course broadcast does not exist in IPv6 */
+    	if((m_strType == "unicast") || (m_strType == "broadcast"))
+      		m_cSocket.SetOption(IPPROTO_IPV6, IPV6_UNICAST_HOPS,
+                                    (char *)&m_iTTL, sizeof(m_iTTL));
+	    else if(m_strType == "multicast")
+      		m_cSocket.SetOption(IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+                                    (char *)&m_iTTL, sizeof(m_iTTL));
+	}
+#endif
 #endif
   }
 
@@ -176,40 +213,78 @@ void C_NetOutput::OnClose()
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_NetOutput::WriteToPort()
+void C_NetOutput::WriteToPort(bool RtpEncapsulation, u32 RtpSendTime)
 {
   ASSERT(m_pTsProvider);
 
   unsigned int iPacketNumber = m_cTsBuff.Size();
+
+  int HeaderOffset;
+  bool HasDiscontinuity = 0;
   
   if(iPacketNumber > 0)
   {
     // To avoid problems with the socket buff
     ASSERT(iPacketNumber <= TS_IN_ETHER);
 
+    if (RtpEncapsulation)
+    {
+      HeaderOffset = RTP_HEADER_LEN;
+      m_pRtpHeader->BuildHeader(m_iRtpCounter++);
+
+      // Check for discontinuity in one of the TS packets
+      for(unsigned int iIndex = 0; iIndex < iPacketNumber; iIndex++)
+        HasDiscontinuity |= m_cTsBuff[iIndex].IsDiscontinuity();
+      m_pRtpHeader->SetRtpDiscontinuity(HasDiscontinuity);
+
+      // Convert to the 90kHzClock : RtpSendTime()*90 000/1 000 000
+      // Doing a /10 *9 /10 fo conversion / integer issues.
+      m_pRtpHeader->SetRtpTimeStamp( ((RtpSendTime/10)*9)/10 );
+
+#ifdef BUGGY_VLC
+      memcpy(m_ByteBuff, *m_pRtpHeader, RTP_HEADER_LEN);
+#endif
+    }
+    else HeaderOffset = 0;
+
 #ifdef BUGGY_VLC
     // TS concatenation
     for(unsigned int iIndex = 0; iIndex < iPacketNumber; iIndex++)
-      memcpy(m_ByteBuff + TS_PACKET_LEN * iIndex, m_cTsBuff[iIndex],
+      memcpy(m_ByteBuff + HeaderOffset + TS_PACKET_LEN * iIndex, m_cTsBuff[iIndex],
              TS_PACKET_LEN);
 
     // Send the data that were stored in the buffer
-    int iRc = PrivateWriteTo(TS_PACKET_LEN * iPacketNumber);
+    int iRc = PrivateWriteTo(HeaderOffset + TS_PACKET_LEN * iPacketNumber);
 
-    if(iRc != TS_IN_ETHER * TS_PACKET_LEN)
-      m_iByteLost += TS_IN_ETHER * TS_PACKET_LEN - iRc;
+    if(iRc != HeaderOffset + TS_IN_ETHER * TS_PACKET_LEN)
+      m_iByteLost += HeaderOffset + TS_IN_ETHER * TS_PACKET_LEN - iRc;
 #else
     // Fill in the socketbuff
-    for(unsigned int iIndex = 0; iIndex < iPacketNumber; iIndex++)
-      m_cSocketBuff.SetSlotBuff(iIndex, m_cTsBuff[iIndex]);
-  
+
+    if (RtpEncapsulation)
+    { // Rtp Header First
+      m_cSocketBuff.SetSlotBuff(0,*m_pRtpHeader);
+    }
+    for(unsigned int iIndex = 1; iIndex < iPacketNumber+1; iIndex++)
+    {
+      m_cSocketBuff.SetSlotBuff(iIndex, m_cTsBuff[iIndex-1]);
+    }
+
     // Send the data that were stored in the buffer
     try
     {
-      int iRc = m_cSocket.Send(m_cSocketBuff, iPacketNumber);
-
-      if(iRc != TS_IN_ETHER * TS_PACKET_LEN)
-        m_iByteLost += TS_IN_ETHER * TS_PACKET_LEN - iRc;
+      int iRc;
+      if (RtpEncapsulation)
+      {
+        iRc = m_cSocket.Send(m_cSocketBuff, iPacketNumber + 1 , 0);
+      }
+      else
+      {
+        // No Rtp : skip first SockBuff -> iOffset=1
+        iRc = m_cSocket.Send(m_cSocketBuff, iPacketNumber, 1);
+      }
+      if(iRc != HeaderOffset + TS_IN_ETHER * TS_PACKET_LEN)
+        m_iByteLost += HeaderOffset + TS_IN_ETHER * TS_PACKET_LEN - iRc;
     }
     catch(E_Exception e)
     {
@@ -230,7 +305,6 @@ void C_NetOutput::WriteToPort()
   }
 }
 
-
 //******************************************************************************
 // C_Net4Output class
 //******************************************************************************
@@ -279,7 +353,7 @@ void C_Net4Output::OnInit()
       imr.imr_interface.s_addr = INADDR_ANY;
       cAddr.Build(m_strDstHost, m_strDstPort);
       imr.imr_multiaddr.s_addr = cAddr.GetInetAddr()->sin_addr.s_addr;
-      m_cSocket.SetOption(IPPROTO_IP, IP_ADD_MEMBERSHIP, &imr, sizeof(imr));
+      m_cSocket.SetOption(IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&imr, sizeof(imr));
     }
 #endif
   }
@@ -306,7 +380,7 @@ void C_Net4Output::OnClose()
       imr.imr_interface.s_addr = INADDR_ANY;
       cAddr.Build(m_strDstHost, m_strDstPort);
       imr.imr_multiaddr.s_addr = cAddr.GetInetAddr()->sin_addr.s_addr;
-      m_cSocket.SetOption(IPPROTO_IP, IP_DROP_MEMBERSHIP, &imr, sizeof(imr));
+      m_cSocket.SetOption(IPPROTO_IP, IP_DROP_MEMBERSHIP, (char *)&imr, sizeof(imr));
     }
 #endif
 
@@ -325,7 +399,7 @@ void C_Net4Output::OnClose()
 //------------------------------------------------------------------------------
 int C_Net4Output::PrivateWriteTo(int iBuffLen)
 {
-  return m_cSocket.WriteTo(m_cOutputInetAddr, m_ByteBuff, iBuffLen);
+  return m_cSocket.WriteTo(m_cOutputInetAddr, (char *)m_ByteBuff, iBuffLen);
 }
 #endif
 
@@ -379,7 +453,7 @@ void C_Net6Output::OnInit()
       imr.ipv6mr_interface = 0;
       cAddr.Build(m_strDstHost, m_strDstPort, SOCK_DGRAM);
       imr.ipv6mr_multiaddr = cAddr.GetInetAddr()->sin6_addr;
-      m_cSocket.SetOption(IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, &imr, sizeof(imr));
+      m_cSocket.SetOption(IPPROTO_IPV6, IPV6_JOIN_GROUP, (char *)&imr, sizeof(imr));
     }
   }
   catch(E_Exception e)
@@ -404,8 +478,8 @@ void C_Net6Output::OnClose()
       imr.ipv6mr_interface = 0;
       cAddr.Build(m_strDstHost, m_strDstPort, SOCK_DGRAM);
       imr.ipv6mr_multiaddr = cAddr.GetInetAddr()->sin6_addr;
-      m_cSocket.SetOption(IPPROTO_IPV6, IPV6_DROP_MEMBERSHIP,
-                          &imr, sizeof(imr));
+      m_cSocket.SetOption(IPPROTO_IPV6, IPV6_LEAVE_GROUP,
+                          (char *)&imr, sizeof(imr));
     }
 
     C_NetOutput::OnClose();
@@ -423,10 +497,9 @@ void C_Net6Output::OnClose()
 //------------------------------------------------------------------------------
 int C_Net6Output::PrivateWriteTo(int iBuffLen)
 {
-  return m_cSocket.WriteTo(m_cOutputInetAddr, m_ByteBuff, iBuffLen);
+  return m_cSocket.WriteTo(m_cOutputInetAddr, (const char *) m_ByteBuff, iBuffLen);
 }
 #endif
 
-
 #endif
 
